#
#
# Author: SEPP HOCHREITER
###############################################################################

setGeneric("showSelected",
           signature = c("object", "which"),
           function(object, which)
           standardGeneric("showSelected"))





setGeneric("parameters", signature = "x",
    function(x) standardGeneric("parameters")
)

setGeneric("parameters<-", signature = c("x", "value"),
    function(x, value) standardGeneric("parameters<-")
)




setGeneric("n", signature = "x",
    function(x) standardGeneric("n")
)

setGeneric("n<-", signature = c("x", "value"),
    function(x, value) standardGeneric("n<-")
)




setGeneric("p1", signature = "x",
    function(x) standardGeneric("p1")
)

setGeneric("p1<-", signature = c("x", "value"),
    function(x, value) standardGeneric("p1<-")
)




setGeneric("p2", signature = "x",
    function(x) standardGeneric("p2")
)

setGeneric("p2<-", signature = c("x", "value"),
    function(x, value) standardGeneric("p2<-")
)




setGeneric("l", signature = "x",
    function(x) standardGeneric("l")
)

setGeneric("l<-", signature = c("x", "value"),
    function(x, value) standardGeneric("l<-")
)




setGeneric("center", signature = "x",
    function(x) standardGeneric("center")
)

setGeneric("center<-", signature = c("x", "value"),
    function(x, value) standardGeneric("center<-")
)


setGeneric("scaleData", signature = "x",
    function(x) standardGeneric("scaleData")
)

setGeneric("scaleData<-", signature = c("x", "value"),
    function(x, value) standardGeneric("scaleData<-")
)




setGeneric("X", signature = "x",
    function(x) standardGeneric("X")
)

setGeneric("X<-", signature = c("x", "value"),
    function(x, value) standardGeneric("X<-")
)




setGeneric("L", signature = "x",
    function(x) standardGeneric("L")
)

setGeneric("L<-", signature = c("x", "value"),
    function(x, value) standardGeneric("L<-")
)




setGeneric("Z", signature = "x",
    function(x) standardGeneric("Z")
)

setGeneric("Z<-", signature = c("x", "value"),
    function(x, value) standardGeneric("Z<-")
)




setGeneric("M", signature = "x",
    function(x) standardGeneric("M")
)

setGeneric("M<-", signature = c("x", "value"),
    function(x, value) standardGeneric("M<-")
)




setGeneric("LZ", signature = "x",
    function(x) standardGeneric("LZ")
)

setGeneric("LZ<-", signature = c("x", "value"),
    function(x, value) standardGeneric("LZ<-")
)




setGeneric("U", signature = "x",
    function(x) standardGeneric("U")
)

setGeneric("U<-", signature = c("x", "value"),
    function(x, value) standardGeneric("U<-")
)



setGeneric("avini", signature = "x",
    function(x) standardGeneric("avini")
)

setGeneric("avini<-", signature = c("x", "value"),
    function(x, value) standardGeneric("avini<-")
)




setGeneric("xavini", signature = "x",
    function(x) standardGeneric("xavini")
)

setGeneric("xavini<-", signature = c("x", "value"),
    function(x, value) standardGeneric("xavini<-")
)




setGeneric("ini", signature = "x",
    function(x) standardGeneric("ini")
)

setGeneric("ini<-", signature = c("x", "value"),
    function(x, value) standardGeneric("ini<-")
)




setGeneric("Psi", signature = "x",
    function(x) standardGeneric("Psi")
)

setGeneric("Psi<-", signature = c("x", "value"),
    function(x, value) standardGeneric("Psi<-")
)




setGeneric("lapla", signature = "x",
    function(x) standardGeneric("lapla")
)

setGeneric("lapla<-", signature = c("x", "value"),
    function(x, value) standardGeneric("lapla<-")
)


